import 'dart:convert';

import 'package:models_weebi/common.dart';
import 'package:models_weebi/utils.dart' show DateRange, WeebiDates;
import 'package:models_weebi/src/closings/closing_ledger.dart';
import 'package:models_weebi/src/closings/closing_range.dart';

extension FinFlowsClosingLedgerShop on List<ClosingLedgerShop> {
  List<FinFlow> shopCLFinFlows(DateRange dateRange, List<FinFlow> flows,
      {String shopUuid = ''}) {
    var closingsFilteredById = this;
    if (shopUuid.isNotEmpty) {
      closingsFilteredById = where(
        (c) => c.shopUuid == shopUuid,
      ).toList();
    }
    closingsFilteredById.forEach((c) {
      if ((c.closingRange.start.isAfter(dateRange.start) ||
              c.closingRange.start.isAtSameMomentAs(dateRange.start)) &&
          (c.closingRange.end.isBefore(dateRange.end) ||
              c.closingRange.start.isAtSameMomentAs(dateRange.start))) {
        flows.sell.sumClosings += c.sell;
        flows.sellCovered.sumClosings += c.sellCovered;
        flows.sellDeferred.sumClosings += c.sellDeferred;
        flows.spend.sumClosings += c.spend;
        flows.spendCovered.sumClosings += c.spendCovered;
        flows.spendDeferred.sumClosings += c.spendDeferred;
        flows.wage.sumClosings += c.wage;
      }
    });
    return flows;
  }
}

/// this model is redundant with closingledger in weebi
/// it is meant to be used when handling several stores
class ClosingLedgerShop extends ClosingLedger {
  bool get isAllZero =>
      sell == 0 &&
      sellCovered == 0 &&
      sellDeferred == 0 &&
      spend == 0 &&
      spendDeferred == 0 &&
      spendCovered == 0 &&
      wage == 0;
  String shopUuid;
  ClosingLedgerShop(super.closingRange, this.shopUuid,
      {required super.sell,
      required super.sellCovered,
      required super.sellDeferred,
      required super.spend,
      required super.spendCovered,
      required super.spendDeferred,
      required super.wage});

  static final dummyMar2020 = ClosingLedgerShop(
    ClosingRange(
      date: DateTime.now(),
      start: WeebiDates.marStart,
      end: WeebiDates.marEnd,
    ),
    'pierre_entrepot',
    sell: 0,
    sellCovered: 0,
    sellDeferred: 200,
    spend: 0,
    spendCovered: 0,
    spendDeferred: 0,
    wage: 0,
  );

  @override
  bool operator ==(other) => identical(closingRange, other);
  @override
  int get hashCode => closingRange.hashCode;

  factory ClosingLedgerShop.fromJson(Map<String, dynamic> map) {
    return ClosingLedgerShop(
      ClosingRange.fromMap(map['closingRange'] as Map<String, dynamic>),
      map['shopUuid'] as String,
      sell: map['sell'] as int,
      sellCovered: map['sellCovered'] as int,
      sellDeferred: map['sellDeferred'] as int,
      spend: map['spend'] as int,
      spendCovered: map['spendCovered'] as int,
      spendDeferred: map['spendDeferred'] as int,
      wage: map['wage'] as int,
    );
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'closingRange': closingRange.toMap(),
      'shopUuid': shopUuid,
      'sell': sell,
      'sellCovered': sellCovered,
      'sellDeferred': sellDeferred,
      'spend': spend,
      'spendCovered': spendCovered,
      'spendDeferred': spendDeferred,
      'wage': wage
    };
  }
}
