import 'dart:convert';
// import 'package:http/http.dart' as http;
import 'package:models_weebi/base.dart' show HerderAbstract;
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/src/extensions/string_no_accents.dart';

class Gender {
  final String _gender;
  const Gender._(this._gender);

  @override
  String toString() => _gender;

  static const Gender woman = Gender._('woman');
  static const Gender man = Gender._('man');
  static const Gender other = Gender._('other');
  static const Gender unknown = Gender._('unknown');

  static Gender tryParse(String val) {
    switch (val) {
      case 'woman':
        return Gender.woman;
      case 'man':
        return Gender.man;
      case 'other':
        return Gender.other;
      case 'unknown':
        return Gender.unknown;
      default:
        print('$val is not a valid gender');
        return Gender.unknown;
    }
  }

  static String genderString(Gender gender) {
    if (gender == Gender.woman) {
      return 'Femme';
    } else if (gender == Gender.man) {
      return 'Homme';
    } else if (gender == Gender.other) {
      return 'Autre';
    } else if (gender == Gender.unknown) {
      return 'Inconnu';
    } else {
      return 'Inconnu';
    }
  }
}

class Herder extends HerderAbstract {
  // final String shopId;
  // String get shopUuid => shopId;

  String get fullName => '$firstName $lastName';
  int get fullNameHash => (firstName.withoutAccents.toLowerCase().trim() +
          lastName.withoutAccents.toLowerCase().trim())
      .hashCode;
  int get mailHash => mail.withoutAccents.toLowerCase().trim().hashCode;
  int get telHash => tel.toLowerCase().trim().hashCode;

  // TODO move away from Herder to Contact
//TODO  faire évoluer isWoman en sex ou gender, avec man/woman/other/unknown
  // Gender gender;
  Herder({
    required super.id,
    required super.bidon,
    required super.firstName,
    // required this.shopId,
    required super.lastName,
    required super.updateDate,
    required super.statusUpdateDate,
    required super.status,
    required super.isWoman,
    // this.gender = Gender.unknown,
    super.area = '',
    super.bank = '',
    super.identity = '',
    super.category = '',
    super.carteNFC = '',
    super.pointCollecte = '',
    super.qrcode = '',
    super.tel,
    super.mail,
    super.address,
    super.avatar,
    super.overdraft,
    super.milkMonthQuota = 0,
  });

  String get sharableText {
    final sb = StringBuffer()
      ..writeln('prenom : $firstName')
      ..writeln('nom : $lastName')
      ..writeln('tel : $tel');
    return sb.toString();
  }

  // only override if add specific field
  // @override Map<String, dynamic> toMap()

  static final dummy = Herder(
    id: 1,
    // shopId: 'dummy',
    bidon: 1,
    firstName: 'John',
    lastName: 'Doe',
    tel: '',
    mail: 'john@doe.com',
    updateDate: WeebiDates.defaultDate,
    statusUpdateDate: WeebiDates.defaultDate,
    status: true,
    isWoman: false,
  );

  static final dummy2 = Herder(
    id: 2,
    // shopId: 'dummy',
    bidon: 2,
    firstName: 'Jimmy',
    lastName: 'Doe',
    tel: '0780086476',
    mail: 'john@doe.com',
    updateDate: WeebiDates.defaultDate,
    statusUpdateDate: WeebiDates.defaultDate,
    status: true,
    isWoman: false,
  );

  static final defaultHerder = Herder(
      id: 0,
      bidon: 0,
      firstName: 'inconnu',
      lastName: '',
      avatar: '',
      address: '',
      area: '',
      bank: '',
      isWoman: true,
      identity: '',
      category: '',
      mail: '',
      tel: '',
      qrcode: '',
      overdraft: 0,
      updateDate: DateTime.fromMillisecondsSinceEpoch(1565888474278),
      status: true,
      statusUpdateDate: DateTime.fromMillisecondsSinceEpoch(1565888474278));

  @override
  String toJson() => json.encode(toMap());

  factory Herder.fromMap(Map<String, dynamic> map) {
    return Herder(
      id: map['id'],
      // shopId: map['shopId'] ?? 'no_shopId',
      bidon: map['bidon'],
      firstName: map['firstName'],
      lastName: map['lastName'],
      tel: map['tel'],
      mail: map['mail'],
      address: map['address'],
      avatar: map['avatar'],
      updateDate: map['updateDate'] == null
          ? WeebiDates.defaultDate
          : DateTime.parse(map['updateDate']),
      statusUpdateDate: map['statusUpdateDate'] == null
          ? WeebiDates.defaultDate
          : DateTime.parse(map['statusUpdateDate']),
      status: map['status'],
      overdraft: map['overdraft'],
      // gender: Gender.tryParse(map['gender'] as String),
      area: map['area'],
      bank: map['bank'],
      identity: map['identity'],
      category: map['category'],
      qrcode: map['qrcode'],
      milkMonthQuota: map['milkMonthQuota'],
      isWoman: map['isWoman'],
      carteNFC: map['carteNFC'],
      pointCollecte: map['pointCollecte'],
    );
  }
  factory Herder.fromJson(String source) => Herder.fromMap(json.decode(source));

  Herder copyWith({
    int? id,
    String? shopId,
    int? bidon,
    String? firstName,
    String? lastName,
    DateTime? updateDate,
    DateTime? statusUpdateDate,
    bool? status,
    bool? isWoman,
    Gender? gender,
    String? area,
    String? bank,
    String? identity,
    String? category,
    String? qrcode,
    String? tel,
    String? mail,
    String? address,
    String? avatar,
    int? overdraft,
    int? milkMonthQuota,
    String? carteNFC,
    String? pointCollecte,
  }) {
    return Herder(
      id: id ?? this.id,
      // shopId: shopId ?? this.shopId,
      bidon: bidon ?? this.bidon,
      firstName: firstName ?? this.firstName,
      lastName: lastName ?? this.lastName,
      updateDate: updateDate ?? this.updateDate,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      status: status ?? this.status,
      tel: tel ?? this.tel,
      mail: mail ?? this.mail,
      address: address ?? this.address,
      avatar: avatar ?? this.avatar,
      overdraft: overdraft ?? this.overdraft,
      milkMonthQuota: milkMonthQuota ?? this.milkMonthQuota,
      area: area ?? this.area,
      bank: bank ?? this.bank,
      identity: identity ?? this.identity,
      category: category ?? this.category,
      qrcode: qrcode ?? this.qrcode,
      isWoman: isWoman ?? this.isWoman,
      // gender: gender ?? this.gender,
      carteNFC: carteNFC ?? this.carteNFC,
      pointCollecte: pointCollecte ?? this.pointCollecte,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is HerderAbstract &&
        other.id == id &&
        other.firstName == firstName &&
        other.lastName == lastName;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        area.hashCode ^
        bank.hashCode ^
        identity.hashCode ^
        category.hashCode ^
        qrcode.hashCode ^
        milkMonthQuota.hashCode ^
        isWoman.hashCode ^
        // gender.hashCode ^
        carteNFC.hashCode ^
        pointCollecte.hashCode;
  }
}
