// Package imports:
import 'package:models_weebi/src/closings/closing_ledger_herder.dart';
import 'package:models_weebi/src/closings/closing_ledger_shop.dart';
import 'package:models_weebi/src/common/fin_flow.dart';
import 'package:models_weebi/src/extensions/tickets_iterable_fin_stats_list.dart';
import 'package:models_weebi/src/reports/report_abstracts.dart';
import 'package:models_weebi/src/models/utils/date_range.dart';

// Project imports:

mixin ComputeFinancialsShop on ReportFinancial<ClosingLedgerShop> {
  List<FinFlow> get financialFlows {
    final dateRange = DateRange(start, end);
    for (var element in emptyFlows) {
      element.sumClosings = 0;
      element.sumTickets = 0;
    }
    emptyFlows = tickets.shopTkFinFlows(dateRange, emptyFlows);
    emptyFlows = closingsObjects.shopCLFinFlows(dateRange, emptyFlows);
    return emptyFlows;
  }
}

mixin ComputeFinancialsHerder on ReportFinancial<ClosingLedgerHerder> {
  List<FinFlow> get financialFlows {
    final dateRange = DateRange(start, end);
    for (var element in emptyFlows) {
      element.sumClosings = 0;
      element.sumTickets = 0;
    }
    if (emptyFlows.any((element) => element.type == FinFlow.balance)) {
      emptyFlows.removeWhere((element) => element.type == FinFlow.balance);
    }

    emptyFlows = tickets.herderTkFinFlows(objectId, dateRange, emptyFlows);
    emptyFlows =
        closingsObjects.herderClFinFlows(objectId, dateRange, emptyFlows);
    final sumtick = (emptyFlows.sellCovered.sumTickets -
            emptyFlows.sellDeferred.sumTickets) -
        (emptyFlows.spendCovered.sumTickets -
            emptyFlows.spendDeferred.sumTickets) -
        emptyFlows.wage.sumTickets;
    final sumcl = (emptyFlows.sellCovered.sumClosings -
            emptyFlows.sellDeferred.sumClosings) -
        (emptyFlows.spendCovered.sumClosings -
            emptyFlows.spendDeferred.sumClosings) -
        emptyFlows.wage.sumClosings;
    // balance is handmade
    final finBalance = FinFlow(
      FinFlow.balance,
      sumTickets: sumtick,
      sumClosings: sumcl,
    );
    emptyFlows.add(finBalance);

    return emptyFlows;
  }
}
