// Package imports:
import 'package:models_weebi/src/closings/closing_stock_herder.dart';
import 'package:models_weebi/src/closings/closing_stock_shop/closing_stock_shop.dart';
import 'package:models_weebi/src/reports/report_stock_article_calibre.dart';
import 'package:models_weebi/src/extensions/tickets_stock_cl_report.dart';
import 'package:models_weebi/src/reports/report_abstracts.dart';
import 'package:models_weebi/src/models/utils/date_range.dart';

mixin ComputeStockShop on ReportStockAbstract<ClosingStockShop> {
  List<ReportStockProduct> get reportStockProducts {
    // reset
    for (final r in emptyReportStockProducts) {
      r.initialQtCl = 0.0;
      r.finalQtCl = 0.0;

      //r.initialQtTickets = 0.0; // used in extension keep it
      r.varQtTickets = 0.0; // used in extension keep it

      for (final a in r.articles) {
        a.qtIn = 0.0;
        a.qtOut = 0.0;
      }
    }

    //print('startDate $startDate  endDate $endDate');
    final dateRange = DateRange(start, end);

    emptyReportStockProducts = tickets.shopTicketsStock(
        objectId, dateRange, articlesC, emptyReportStockProducts);

    if (closingsObjects.isNotEmpty) {
      for (final c in closingsObjects) {
        if (c.shopUuid == objectId && c.closingRange.isInRange(start, end)) {
          articlesC
              .where((p) => p.id != 0) // not including product 0
              .forEach((p) {
            if (c.products.any((i) => i.id == p.id)) {
              // find the one matching the time filter
              final _ClStPrd = c.products
                  .firstWhere((closingStockP) => closingStockP.id == p.id);

              var rP = emptyReportStockProducts
                  .firstWhere((reportStockP) => reportStockP.id == p.id);
              // * here one way only thanks to final Quantity
              rP.initialQtCl += _ClStPrd.initialQtCl;
              rP.finalQtCl += _ClStPrd.finalQtCl;

              if (p.articles.length > 1) {
                for (final article in p.articles) {
                  // find the one matching the time filter
                  if (_ClStPrd.articles.any((a) =>
                      a.id == article.id && a.productId == article.productId)) {
                    final cSArticle = _ClStPrd.articles.firstWhere((a) =>
                        a.id == article.id && a.productId == article.productId);

                    var aR = rP.articles.firstWhere((a) =>
                        a.id == article.id && a.productId == article.productId);

                    aR.qtIn += cSArticle.quantityIn;
                    aR.qtOut += cSArticle.quantityOut;
                  }
                }
              }
            }
          });
        }
      }
    }
    return emptyReportStockProducts;
  }
}

mixin ComputeStockHerder on ReportStockAbstract<ClosingStockHerder> {
  List<ReportStockProduct> get reportStockProducts {
    // reset
    for (final r in emptyReportStockProducts) {
      r.initialQtCl = 0.0;
      r.finalQtCl = 0.0;
      //r.initialQtTickets = 0.0; // not mentioned here but needed keep it
      r.varQtTickets = 0.0; // not mentioned here but needed keep it
      for (final a in r.articles) {
        a.qtIn = 0.0;
        a.qtOut = 0.0;
      }
    }
    final dateRange = DateRange(start, end);
    // ticket logic is handled here
    emptyReportStockProducts = tickets.herderTicketsStock(
        objectId, dateRange, articlesC, emptyReportStockProducts);

    final _products = articlesC.where((p) => p.id != 0);
    for (final cH in closingsObjects) {
      // filtering like .where but more performant
      if (cH.herderId == objectId && cH.closingRange.isInRange(start, end)) {
        for (final p in _products) {
          // NO products can be bought & sold on the same ticket
          // so we can SUM ALL movements

          if (cH.products.any((cSP) => cSP.id == p.id)) {
            // qtin && qtOut needed for milk

            // find the one matching the time filter
            final _clStockPrd = cH.products
                .firstWhere((closingStockP) => closingStockP.id == p.id);
            // find the report
            final rP = emptyReportStockProducts
                .firstWhere((reportStockP) => reportStockP.id == p.id);
            // update values
            rP.initialQtCl += _clStockPrd.initialQtCl;
            rP.finalQtCl += _clStockPrd.finalQtCl;
            // print("rP.finalQtCl ${rP.finalQtCl} rP.initialQtCl ${rP.initialQtCl} ${rP.title}");
            if (p.articles.length > 1) {
              for (final article in p.articles) {
                // find the one matching the time filter
                if (_clStockPrd.articles.any((a) =>
                    a.id == article.id && a.productId == article.productId)) {
                  // find the exact closing
                  var _cSArticle = _clStockPrd.articles.firstWhere((a) =>
                      a.id == article.id && a.productId == article.productId);

                  // find the report
                  final aR = rP.articles.firstWhere((element) =>
                      element.id == article.id &&
                      element.productId == article.productId);
                  // update values
                  aR.qtIn += _cSArticle.quantityIn;
                  aR.qtOut += _cSArticle.quantityOut;
                }
              }
            }
          } else {
            // print('closings do not fit requirements');
          }
        }
      }
    }
    return emptyReportStockProducts;
  }
}
