library report_abstract;

// Flutter imports:
import 'package:models_weebi/src/closings/abstract/closing_range.dart';
import 'package:models_weebi/src/reports/report_stock_article.dart';
import 'package:models_weebi/src/reports/report_stock_article_calibre.dart';
import 'package:models_weebi/src/common/fin_flow.dart';
import 'package:models_weebi/src/common/ticket_type.dart';
import 'package:models_weebi/src/models/article_calibre.dart';
import 'package:models_weebi/src/models/ticket_weebi.dart';
import 'package:models_weebi/src/models/utils/date_range.dart';

// Project imports:

abstract class Report<C extends ClosingRangeAbstract> implements DateRange {
  final List<TicketWeebi> tickets;
  final List<C> closingsObjects;
  @override
  final DateTime start;
  @override
  final DateTime end;
  const Report(
    this.tickets,
    this.closingsObjects,
    this.start,
    this.end,
  );
}

class CellTotal {
  final DateRange dateRange;
  final TicketType ticketType;
  final double total;
  const CellTotal(this.dateRange, this.ticketType, this.total);
}

abstract class ReportFinancial<C extends ClosingRangeAbstract>
    extends Report<C> {
  // this could be used for distinguising shops
  // not useful here for weebi with singlepos
  final String objectId;
  List<FinFlow> emptyFlows;
  ReportFinancial({
    required List<TicketWeebi> tickets,
    required List<C> closingsObjects,
    required DateTime startDate,
    required DateTime endDate,
    required this.objectId,
  })  : emptyFlows = List<FinFlow>.generate(
          TicketType.financialTypes.length,
          (index) => FinFlow(
            TicketType.financialTypes[index].toString(),
            sumTickets: 0,
            sumClosings: 0,
          ),
        ),
        super(
          tickets,
          closingsObjects,
          startDate,
          endDate,
        );
}

abstract class ReportStockAbstract<C extends ClosingRangeAbstract>
    extends Report<C> {
  final String objectId;
  List<ReportStockProduct> emptyReportStockProducts;
  final List<ArticleCalibre> articlesC;
  ReportStockAbstract({
    required List<TicketWeebi> tickets,
    required List<C> closingsObjects,
    required DateTime startDate,
    required DateTime endDate,
    required this.objectId,
    required this.articlesC,
  })  : emptyReportStockProducts = List.generate(articlesC.length, (index) {
          final rAs = <ReportStockArticle>[];
          for (final a in articlesC[index].articles) {
            final rA = ReportStockArticle(
                productId: a.productId,
                id: a.id,
                start: startDate,
                end: endDate,
                initialQtCl: 0.0,
                finalQtCl: 0.0,
                qtIn: 0.0,
                qtOut: 0.0,
                fullName: a.fullName);
            rAs.add(rA);
          }
          return ReportStockProduct(
              id: articlesC[index].id,
              title: articlesC[index].title,
              start: startDate,
              end: endDate,
              initialQtCl: 0.0,
              finalQtCl: 0.0,
              varQtTickets: 0.0,
              // initialQtTickets: 0.0,
              articles: rAs);
        }),
        super(
          tickets,
          closingsObjects,
          startDate,
          endDate,
        );
}

class Milk {
  int day;
  double volTkCvo;
  double volTkNoCvo;
  double totalTk;
  double volClCvo;
  double volClNoCvo;
  double totalCl;
  double total;
  Milk(this.day, this.volTkCvo, this.volTkNoCvo, this.volClCvo, this.volClNoCvo)
      : totalTk = volTkCvo + volTkNoCvo,
        totalCl = volClCvo + volClNoCvo,
        total = volTkCvo + volTkNoCvo + volClCvo + volClNoCvo;
}

abstract class ReportMilkAbstract<C extends ClosingRangeAbstract>
    extends Report<C> {
  final String objectId;
  List<Milk> get flows;
  ReportMilkAbstract({
    required List<TicketWeebi> tickets,
    required List<C> closingsObjects,
    required this.objectId,
    required DateTime startDate,
    required DateTime endDate,
  }) : super(
          tickets,
          closingsObjects,
          startDate,
          endDate,
        );
}
