library report_financials;

import 'package:models_weebi/common.dart';
import 'package:models_weebi/extensions.dart';
import 'package:models_weebi/src/closings/closing_ledger_herder.dart';
import 'package:models_weebi/src/closings/closing_ledger_shop.dart';
import 'package:models_weebi/src/models/herder.dart';
import 'package:models_weebi/src/models/shop_weebi.dart';
import 'package:models_weebi/src/models/ticket_weebi.dart';
import 'package:models_weebi/src/reports/compute_financials_mixin.dart';
import 'package:models_weebi/src/reports/report_abstracts.dart';
import 'package:models_weebi/utils.dart';

class ReportFinancialHerder extends ReportFinancial<ClosingLedgerHerder>
    with ComputeFinancialsHerder {
  final Herder herder;
  ReportFinancialHerder({
    required this.herder,
    required List<ClosingLedgerHerder> closingHerderLedgers,
    required super.tickets,
    required super.startDate,
    required super.endDate,
  }) : super(
          objectId: '${herder.id}',
          closingsObjects: closingHerderLedgers,
        );
}

extension AccountingCharts on Iterable<ReportFinancialShop> {
  double get cashFlowAverage =>
      (fold(0.0, (pv, e) => pv + e.financialFlows.cashflow) / length)
          .roundTwoDecimals;

  Map<TicketType, double> get totalPerType {
    final map = <TicketType, double>{};
    for (final report in this) {
      for (final ticketType in TicketType.financialTypes) {
        if (map[ticketType] != null) {
          map[ticketType] = ((map[ticketType])! +
              report.financialFlows.sumTotalForType(ticketType));
        } else {
          map[ticketType] = report.financialFlows.sumTotalForType(ticketType);
        }
      }
    }
    return map;
  }
}

extension PutReportFinancialInADatatable on Iterable<ReportFinancialShop> {
  Map<TicketType, Map<DateRange, double>> get cellsTotalMap {
    final temp = <CellTotal>[];
    for (final report in this) {
      for (final ticketType in TicketType.financialTypes) {
        temp.add(
          CellTotal(
            DateRange(report.start, report.end),
            ticketType,
            report.financialFlows.sumTotalForType(ticketType),
          ),
        );
      }
    }
    final rows = <TicketType, Map<DateRange, double>>{};
    for (final fin in TicketType.financialTypes) {
      rows[fin] = {};
    }
    for (final row in rows.entries) {
      final cells = <DateRange, double>{};
      for (final e in temp.where((element) => element.ticketType == row.key)) {
        cells[e.dateRange] = e.total;
      }
      row.value.addAll(cells);
    }

    return rows;
  }
}

class ReportFinancialShop extends ReportFinancial<ClosingLedgerShop>
    with ComputeFinancialsShop {
  final String shopId;
  ReportFinancialShop({
    required this.shopId,
    required List<ClosingLedgerShop> closingLedgerShops,
    required super.tickets,
    required super.startDate,
    required super.endDate,
  }) : super(
          objectId: shopId,
          closingsObjects: closingLedgerShops,
        );
  static final dummy = ReportFinancialShop(
      shopId: ShopWeebi.dummy.uuid,
      tickets: [TicketWeebi.dummySell],
      closingLedgerShops: [],
      startDate: WeebiDates.defaultFirstDate.add(const Duration(days: 1)),
      endDate: WeebiDates.defaultLastDate.subtract(const Duration(days: 1)));
}
