import 'package:models_weebi/extensions.dart' show Day, Week, Month, Year;
import 'package:models_weebi/utils.dart';
import 'package:test/test.dart';

void main() {
  // by default we will display last 7 days
  // by default we will display last 4 weeks
  // by default we will display last 12 months
  // by default we will display last 3 years

  group('test default dateranges', () {
    test('last 7 days', () {
      final list = WeebiDates.sunday19Nov2023.lastSevenDays;
      final sunday = list.first;
      final saturday = list[1];
      final twoDaysAgo = list[2];
      final threeDaysAgo = list[3];
      final fourDaysAgo = list[4];
      final fiveDaysAgo = list[5];
      final sixDaysAgo = list[6];

      expect(sunday.toString(),
          '2023-11-19T00:00:00.000001 - 2023-11-19T23:59:59.999999');
      expect(saturday.toString(),
          '2023-11-18T00:00:00.000001 - 2023-11-18T23:59:59.999999');
      expect(twoDaysAgo.toString(),
          '2023-11-17T00:00:00.000001 - 2023-11-17T23:59:59.999999');
      expect(threeDaysAgo.toString(),
          '2023-11-16T00:00:00.000001 - 2023-11-16T23:59:59.999999');
      expect(fourDaysAgo.toString(),
          '2023-11-15T00:00:00.000001 - 2023-11-15T23:59:59.999999');
      expect(fiveDaysAgo.toString(),
          '2023-11-14T00:00:00.000001 - 2023-11-14T23:59:59.999999');
      expect(sixDaysAgo.toString(),
          '2023-11-13T00:00:00.000001 - 2023-11-13T23:59:59.999999');
    });
    test('last 4 weeks', () {
      final list = WeebiDates.sunday19Nov2023.lastFourWeeks;

      final thisWeek = list.first;
      final lastWeek = list[1];
      final twoWeeksAgo = list[2];
      final threeWeeksAgo = list.last;

      expect(thisWeek.toString(),
          '2023-11-13T00:00:00.000001 - 2023-11-19T23:59:59.999999');
      expect(lastWeek.toString(),
          '2023-11-06T00:00:00.000001 - 2023-11-12T23:59:59.999999');
      expect(twoWeeksAgo.toString(),
          '2023-10-30T00:00:00.000001 - 2023-11-05T23:59:59.999999');
      expect(threeWeeksAgo.toString(),
          '2023-10-23T01:00:00.000001 - 2023-10-29T23:59:59.999999');
    });

    test('past 12+1 months', () {
      final list = WeebiDates.sunday19Nov2023.lastThirteenMonths;
      final thisMonth = list.first;

      final oneMonthAgo = list[1];
      final twoMonthsAgo = list[2];
      final threeMonthsAgo = list[3];
      final fourMonthsAgo = list[4];
      final fiveMonthsAgo = list[5];
      final sixMonthsAgo = list[6];
      final sevenMonthsAgo = list[7];
      final eightMonthsAgo = list[8];
      final nineMonthsAgo = list[9];
      final tenMonthsAgo = list[10];
      final elevenMonthsAgo = list[11];
      final twelveMonthsAgo = list.last;

      expect(thisMonth.toString(),
          '2023-11-01T00:00:00.000001 - 2023-11-19T23:59:59.999999');
      expect(oneMonthAgo.toString(),
          '2023-10-01T00:00:00.000001 - 2023-10-31T23:59:59.999999');
      expect(twoMonthsAgo.toString(),
          '2023-09-01T00:00:00.000001 - 2023-09-30T23:59:59.999999');
      expect(threeMonthsAgo.toString(),
          '2023-08-01T00:00:00.000001 - 2023-08-31T23:59:59.999999');
      expect(fourMonthsAgo.toString(),
          '2023-07-01T00:00:00.000001 - 2023-07-31T23:59:59.999999');
      expect(fiveMonthsAgo.toString(),
          '2023-06-01T00:00:00.000001 - 2023-06-30T23:59:59.999999');
      expect(sixMonthsAgo.toString(),
          '2023-05-01T00:00:00.000001 - 2023-05-31T23:59:59.999999');
      expect(sevenMonthsAgo.toString(),
          '2023-04-01T00:00:00.000001 - 2023-04-30T23:59:59.999999');
      expect(eightMonthsAgo.toString(),
          '2023-03-01T00:00:00.000001 - 2023-03-31T23:59:59.999999');
      expect(nineMonthsAgo.toString(),
          '2023-02-01T00:00:00.000001 - 2023-02-28T23:59:59.999999');
      expect(tenMonthsAgo.toString(),
          '2023-01-01T00:00:00.000001 - 2023-01-31T23:59:59.999999');
      expect(elevenMonthsAgo.toString(),
          '2022-12-01T00:00:00.000001 - 2022-12-31T23:59:59.999999');
      expect(twelveMonthsAgo.toString(),
          '2022-11-01T00:00:00.000001 - 2022-11-30T23:59:59.999999');
    });

    test('past 3 years', () {
      final list = WeebiDates.sunday19Nov2023.lastThreeYears;
      final thisYear = list.first;
      final previousYear = list[1];
      final twoYearsAgo = list.last;

      expect(thisYear.toString(),
          '2023-01-01T00:00:00.000001 - 2023-12-31T23:59:59.999999');
      expect(previousYear.toString(),
          '2022-01-01T00:00:00.000001 - 2022-12-31T23:59:59.999999');
      expect(twoYearsAgo.toString(),
          '2021-01-01T00:00:00.000001 - 2021-12-31T23:59:59.999999');
    });
  });
}
